import requests

from typing import Literal
from fastapi import APIRouter, HTTPException, Request
from fastapi.responses import (
    Response,
    RedirectResponse,
)

from bat.api.utils import config

router = APIRouter(tags=['Allure'], prefix='/allure')
ALLURE_SERVER = config().allure_server.rstrip('/')


@router.get('/customers')
async def get_customers() -> list[str]:
    request = f'{ALLURE_SERVER}/projects'
    response = requests.get(request)
    json = response.json()
    if response.ok:
        customers = []
        for key, _ in json['data']['projects'].items():
            customers.append(key)
        return customers
    else:
        return HTTPException(
            status_code=response.status_code,
            detail=json['metadata'].get('message'),
        )


@router.get('/reports/{customer}')
async def get_customer(customer: str) -> list[str]:
    request = f'{ALLURE_SERVER}/projects/{customer}'
    response = requests.get(request)
    json = response.json()
    if response.ok:
        return json['data']['project']['reports_id']
    else:
        raise HTTPException(
            status_code=response.status_code,
            detail=json['meta_data'].get('message'),
        )


@router.get('/report/emailable')
async def get_emailable_report(customer: str) -> Response:
    request = f'{ALLURE_SERVER}/emailable-report/render?project_id={customer}'
    response = requests.get(request)
    if response.ok:
        return Response(content=response.content, headers=response.headers)
    else:
        raise HTTPException(
            status_code=response.status_code,
            detail=response.content,
            headers=response.headers,
        )


@router.get('/export/{format}')
async def export_report(
    format: Literal['emailable', 'full'], customer: str
) -> Response:
    if format == 'emailable':
        request = (
            f'{ALLURE_SERVER}/emailable-report/export?project_id={customer}'
        )

    else:
        request = f'{ALLURE_SERVER}/report/export?project_id={customer}'
    response = requests.get(request)
    if response.ok:
        return Response(content=response.content, headers=response.headers)
    else:
        raise HTTPException(
            status_code=response.status_code,
            detail=response.content,
            headers=response.headers,
        )


@router.get('/generate')
async def generate_report(
    request: Request,
    customer: str,
    execution_name: str,
    execution_from: str,
    execution_type: str,
) -> Response:
    req = f'{ALLURE_SERVER}/generate-report'
    params = {
        'project_id': customer,
        'execution_name': execution_name,
        'execution_from': execution_from,
        'execution_type': execution_type,
    }

    response = requests.get(req, params=params)
    if response.ok:
        report_url: str = response.json()['data']['report_url']
        path = f"{report_url.split('/')[-2]}/index.html"
        redirect_url = request.url_for(
            'get_customer_report',
            customer=customer,
            path=path,
        )
        return RedirectResponse(redirect_url)

    else:
        raise HTTPException(
            status_code=response.status_code,
            detail=response.content,
            headers=response.headers,
        )


@router.get('/latest-report')
async def get_latest_report(
    request: Request, customer: str
) -> RedirectResponse:
    redirect = request.url_for(
        'get_customer_report',
        customer=customer,
        path='latest/index.html',
    )
    return RedirectResponse(redirect)


@router.get('/report/{customer}/{id}')
async def render_customer_report(
    request: Request, customer: str, id: str
) -> RedirectResponse:
    redirect = request.url_for(
        'get_customer_report',
        customer=customer,
        path=f'{id}/index.html',
    )
    return RedirectResponse(redirect)


@router.get('/{customer}/{path:path}')
async def get_customer_report(customer: str, path: str) -> Response:
    request = f'{ALLURE_SERVER}/projects/{customer}/reports/{path}'
    response = requests.get(request)
    if response.ok:
        return Response(response.content, headers=response.headers)
    else:
        raise HTTPException(
            status_code=response.status_code,
            detail=response.content,
            headers=response.headers,
        )


@router.post('/email/{customer}')
async def email_report(customer: str) -> Response:
    ...
