import os
import sys
import pytest
import argparse

from pathlib import Path
from bat.core.logger import logger
from bat.libs.utils import write_yaml
from bat.core.definitions import CONFIGS_DIR, LOGS_DIR, TESTS_DIR
from bat.validators.base_config import (
    TestConfig,
    ArgoCDConfig,
    PlatformConfig,
    GitConfig,
)


parser = argparse.ArgumentParser(
    prog='bat', description='Platform Basic Acceptance Tests'
)


class InputAction(argparse.Action):
    def __call__(self, parser, namespace, value, option_string=None):
        setattr(namespace, self.dest, value)


# Hidden #
parser.add_argument('--api', action='store_true', help=argparse.SUPPRESS)
parser.add_argument('--account', help=argparse.SUPPRESS)
parser.add_argument('--git_user', help=argparse.SUPPRESS, action=InputAction)
parser.add_argument(
    '--git_password',
    help=argparse.SUPPRESS,
    action=InputAction,
)
parser.add_argument('-c', '--customer', help=argparse.SUPPRESS)
parser.add_argument('--git_branch', help=argparse.SUPPRESS, action=InputAction)
##########

parser.add_argument(
    "-v",
    "--vishwakarma",
    action="store_true",
    help="Run tests on Platform level",
)
parser.add_argument(
    "-f",
    "--functional",
    action="store_true",
    help="Run tests on Functional level",
)
parser.add_argument(
    "-s", "--scenario", action="store_true", help="Run tests on Scenario level"
)

parser.add_argument('--feed', help='Feed code')
parser.add_argument(
    '-p', '--path', help="Path to customer folder (including name of git repo)"
)
parser.add_argument(
    '--blip-file',
    help='Blip file name, default = blip.yaml',
    default='blip.yaml',
)
parser.add_argument(
    '--account-file',
    help='Account file name, default = account.yaml',
    default='account.yaml',
)
parser.add_argument(
    '--argocd_url',
    help="ArgoCD URL",
    action=InputAction,
)
parser.add_argument(
    '-u',
    '--argocd_user',
    help="ArgoCD username, default = admin",
    default="admin",
)
parser.add_argument(
    '--argocd_password',
    help="ArgoCD password",
    action=InputAction,
)
parser.add_argument(
    '--managed',
    action='store_true',
    help='Amagi managed setup',
)
parser.add_argument('--prod', action='store_true', help='Prod setup')
parser.add_argument('--nbcu', action='store_true', help='NBCU specific setup')
parser.add_argument('--allure_server', help="[Optional] Allure server url")

args = parser.parse_args()
input_str = "\u001b[34;m {string} \x1b[0m"

if args.functional and (args.path is None or args.channel is None):
    parser.error("--component requires --path and --channel")
if args.scenario and (args.path is None or args.channel is None):
    parser.error("--scenario requires --path and --channel")
if not args.argocd_url:
    args.argocd_url = input(input_str.format(string="Enter ArgoCD URL: "))
if not args.argocd_password:
    args.argocd_password = input(
        input_str.format(string="Enter ArgoCD Password: ")
    )


def test_config(args) -> TestConfig:
    if args.api:
        test_config = TestConfig(
            exec_mode='api',
            deploy_blip=False,
            deploy_account=False,
            deploy_player=False,
            existing_blip=args.customer,
            existing_account=args.account,
            existing_channel=args.feed,
        )
        os.environ['CUSTOMER'] = (
            args.customer if args.customer else os.environ['CLUSTER']
        )

    else:
        args.git_user = input(
            input_str.format(string="Enter github username: ")
        )
        args.git_password = input(
            input_str.format(string="Enter github personal access token: ")
        )
        args.git_branch = input(
            input_str.format(
                string='Enter github branch: (leave blank for default = "main")'
            )
        )
        path = Path(args.path).parts
        region_long = {
            "use1": "us-east-1",
            "use2": "us-east-2",
            "usw1": "us-west-1",
            "usw2": "us-west-2",
            "eun1": "eu-north-1",
            "euw1": "eu-west-1",
            "euw3": "eu-west-3",
            "euw2": "eu-west-2",
            "eus1": "eu-south-1",
            "euc1": "eu-central-1",
            "as1": "ap-south-1",
            "ane1": "ap-northeast-1",
            "ane2": "ap-northeast-2",
            "ane3": "ap-northeast-3",
            "ae1": "ap-east-1",
            "ase1": "ap-southeast-1",
            "ase2": "ap-southeast-2",
            "ase3": "ap-southeast-3",
            "afs1": "af-south-1",
            "mes1": "me-south-1",
            "sae1": "sa-east-1",
            "cac1": "ca-central-1",
        }
        test_config = TestConfig(
            exec_mode='cli',
            allure_server=args.allure_server,
            deploy_blip=False,
            deploy_account=False,
            deploy_player=False,
            argocd_config=ArgoCDConfig(
                server=args.argocd_url,
                username=args.argocd_user,
                password=args.argocd_password,
            ),
            existing_blip="-".join(path[6].split('-')[:-1]),
            existing_account=None,
            existing_channel=args.feed,
            platform_config=PlatformConfig(
                enterprise=not args.managed,
                environment='prod' if args.prod else 'dev',
                cp_integration=True if args.nbcu else False,
                cp_orchestration=True if args.nbcu else False,
                cluster=path[5],
                cloud=path[4].split('-')[0],
                region=region_long[path[4].split('-')[1]],
                stage=path[1],
            ),
            git_config=GitConfig(
                repo=path[0],
                user=args.git_user,
                password=args.git_password,
                branch=args.git_branch if args.git_branch else 'main',
                org='amagimedia',
            ),
        )
    return test_config


def pytest_markers(args) -> str:
    markers = []
    if args.vishwakarma:
        markers.append("evt")
    if args.functional:
        markers.append("functional")
    if args.scenario:
        markers.append("e2e")
    if len(markers) == 0:
        logger.error('No test level provided, exiting...')
        sys.exit(1)

    marker_str = " or "
    marker_str = marker_str.join(markers)
    logger.debug("pytest marker args: %s", marker_str)
    return marker_str


def main():
    path = Path(os.path.join(CONFIGS_DIR, 'test.yaml'))
    write_yaml(path, test_config(args).dict(), 'pyyaml')
    report_dir = os.path.join(LOGS_DIR, 'report.html')
    allure_dir = os.path.join(LOGS_DIR, 'allure-results')
    flags = [
        TESTS_DIR,
        f"--html={report_dir}",
        "--self-contained-html",
        f"--alluredir={allure_dir}",
        "-m",
    ]
    flags.append(pytest_markers(args))
    logger.debug("pytest runtime flags: \n%s", " ".join(flags))

    pytest.main(flags)
