import os
import logging

from pathlib import Path
from pydantic import BaseModel
from functools import lru_cache
from rich.logging import RichHandler
from bat.core.definitions import LOGS_DIR


DATE_FORMAT = "%d %b %Y | %H:%M:%S"
LOGGER_FORMAT = "%(asctime)s | %(message)s"


class LoggerConfig(BaseModel):
    handlers: list
    format: str | None
    date_format: str | None = None
    logger_file: Path | None = None
    level: int = logging.DEBUG


@lru_cache
def get_logger_config(file: str, level: int):
    file_path = Path(os.path.join(LOGS_DIR, file))
    output_file_handler = logging.FileHandler(file_path)
    handler_format = logging.Formatter(LOGGER_FORMAT, datefmt=DATE_FORMAT)
    output_file_handler.setFormatter(handler_format)

    return LoggerConfig(
        handlers=[
            RichHandler(
                rich_tracebacks=True,
                tracebacks_show_locals=True,
                show_time=False,
            ),
            output_file_handler,
        ],
        format=None,
        date_format=DATE_FORMAT,
        logger_file=file_path,
        level=level,
    )


class Logger:
    @staticmethod
    def setup_logger(file: str, level: int = logging.INFO):
        for name in logging.root.manager.loggerDict.keys():
            logging.getLogger(name).handlers = []
            logging.getLogger(name).propagate = True

        logger_config = get_logger_config(file=file, level=level)

        logging.basicConfig(
            level=logger_config.level,
            format=logger_config.format,
            datefmt=logger_config.date_format,
            handlers=logger_config.handlers,
        )


logger = logging.getLogger()
