from pathlib import Path
from typing import Literal
from bat.core.logger import logger
from bat.setups._models import SetupResource
from bat.extractors._extractor import BaseExtractor
from bat.extractors._blip.cli import BlipCliExtractor
from bat.extractors._blip.api import BlipApiExtractor
from bat.models.blip.crd import Spec
from bat.models.data import (
    BlipData,
    BlipConfig,
    CpAutomation,
    PubnubConfig,
)


class BlipExtractor(BaseExtractor):
    def __new__(
        cls, path: SetupResource | Path, mode: Literal['api', 'cli']
    ) -> BlipData:
        if mode == 'cli':
            logger.info('Running Blip CLI extractor...')
            spec = BlipCliExtractor(path)
        elif mode == 'api':
            logger.info('Running Blip API extractor...')
            spec = BlipApiExtractor(path)
        else:
            raise NotImplementedError

        blip_data = cls.extract_test_data(spec)
        return blip_data

    @staticmethod
    def extract_test_data(spec: Spec) -> BlipData:
        logger.info('Extracting blip data...')
        return BlipData(
            customer=spec.customer.name,
            root_domain=spec.cp_automation.configs.settings_yml.defaults.root_domain,
            host_name=spec.customer.hostName,
            epub_url=spec.cp_automation.configs.settings_yml.defaults.epub.server,
            cp_automation=CpAutomation(
                release=spec.cp_automation.blip.containerImage.split(":")[-1],
                pubnub=PubnubConfig(
                    publish_key=spec.cp_automation.blip.env.PUBNUB_PUBLISH_KEY,
                    subscriber_key=spec.cp_automation.blip.env.PUBNUB_SUBSCRIBE_KEY,
                    secret_key=spec.cp_automation.blip.env.PUBNUB_SECRET_KEY,
                ),
            ),
            config=BlipConfig(
                nexus_elasticache=spec.config.nexus.plec.config.elasticCacheEndpoint,
                turnservers=spec.config.nexus.plec.config.iceServers,
            ),
        )
