import time
import requests

from bat.core.logger import logger
from requests.exceptions import RequestException
from bat.core.errors import ArgoCDAPIError, PollingTimeout
from bat.models.argocd import (
    _Node,
    DegradedResource,
    Application,
    JWTRequest,
    JWTResponse,
    DefaultError,
    ResourceTree,
)


class ArgoCDClient:
    def __init__(self, server: str, username: str, password: str):
        self.server = server.rstrip('/')
        token = self.__generate_token(
            username,
            password,
        )
        self.header = {'Authorization': f'Bearer {token}'}
        self.__resource_tree = None
        self.__application = None

    @property
    def all_nodes(self) -> list[_Node]:
        if self.__resource_tree:
            return (
                self.__resource_tree.nodes + self.__resource_tree.orphanedNodes
            )
        else:
            return None

    def poll_job_health(
        self,
        job_name: str,
        app_name: str,
        timeout_min: int | None = 2,
        exponent: float | None = 2.0,
    ):
        total_sleep = 1
        poll_num = 1

        while total_sleep < timeout_min * 60:
            time.sleep(pow(poll_num, exponent))
            total_sleep += pow(poll_num, exponent)
            poll_num += 1
            try:
                self.__get_resource_tree(app_name)
                node = self.__get_resource(job_name, 'job')
                if node and node.health.status == 'Healthy':
                    return "Healthy"
                elif node and node.health.status == 'Degraded':
                    return "Degraded"
                elif node:
                    status = node.health.status
                    logger.info('Health status: %s', status)
                else:
                    status = 'Not Found'
                    logger.info('job not found yet...')
            except Exception as e:
                raise e

        logger.exception(
            "Failed after polling for %i minutes: Status - %s",
            timeout_min,
            status,
        )

        raise PollingTimeout(timeout_min)

    def poll_secret_creation(
        self,
        secret: str,
        app_name: str,
        timeout_min: int | None = 2,
        exponent: float = 2.0,
    ):
        total_sleep = 1
        poll_num = 1

        while total_sleep < timeout_min * 60:
            time.sleep(pow(poll_num, exponent))
            total_sleep += pow(poll_num, exponent)
            poll_num += 1
            try:
                self.__get_resource_tree(app_name)
                secret_node = self.__get_resource(secret, 'secret')
                if secret_node:
                    return True
                else:
                    logger.info('Secret not found yet...')
            except Exception as e:
                raise e

        logger.exception(
            "Failed after polling for %i minutes",
            timeout_min,
        )

        raise PollingTimeout(timeout_min)

    def poll_app_health(
        self,
        app_name: str,
        timeout_min: int | None = 2,
        exponent: float = 2.0,
    ):
        total_sleep = 1
        poll_num = 1

        while total_sleep < timeout_min * 60:
            time.sleep(pow(poll_num, exponent))
            total_sleep += pow(poll_num, exponent)
            poll_num += 1
            try:
                self.__get_application(app_name)
                application = self.__application

                if (
                    application
                    and application.status.health.status == 'Healthy'
                ):
                    return 'Healthy'
                elif (
                    application
                    and application.status.health.status == 'Degraded'
                ):
                    return 'Degraded'
                elif application:
                    status = application.status.health.status
                    logger.info('Health status: %s', status)
                else:
                    status = 'Not Found'
                    logger.info('application not found yet')

            except Exception as e:
                raise e

        logger.exception(
            "Failed after polling for %i minutes: Status - %s",
            timeout_min,
            status,
        )

        raise PollingTimeout(timeout_min)

    def __get(
        self,
        endpoint: str,
        header: dict[str, str] | None = None,
        max_retries: int = 3,
    ):
        retries = 0
        while retries < max_retries:
            try:
                response = requests.get(
                    url=endpoint,
                    headers=header,
                    verify=True,
                )
                response.raise_for_status()
                return response
            except (RequestException, ValueError) as e:
                logger.warning(f"Request failed: {str(e)}")
                retries += 1
        return response

    def __generate_token(self, username: str, password: str) -> None:
        try:
            session = self.server + '/api/v1/session'
            payload = JWTRequest(
                password=password,
                username=username,
            )
            logger.info("Generating argocd bearer token")
            response = requests.post(
                url=session,
                json=payload.dict(),
                verify=True,
            )
            if response.status_code == 200:
                jwt = JWTResponse(**response.json())
                return jwt.token
            else:
                error = DefaultError(**response.json())
                logger.error(
                    "Status code: %i, Message: %s",
                    error.code,
                    error.message,
                )
                raise ArgoCDAPIError(error.error)
        except Exception as e:
            raise e

    def __get_resource_tree(self, app_name: str):
        endpoint = (
            self.server + f'/api/v1/applications/{app_name}/resource-tree'
        )
        logger.info('Getting resource tree for %s', app_name)
        response = self.__get(
            endpoint,
            self.header,
        )
        if response.status_code == 200:
            resource_tree = ResourceTree(**response.json())
            self.__resource_tree = resource_tree
        else:
            error = DefaultError(**response.json())
            logger.error(
                "Status code: %i, Message: %s",
                error.code,
                error.message,
            )
            raise ArgoCDAPIError(error.error)

    def __get_resources_by_name(self, name: str):
        logger.info('Getting resource %s', name)
        filtered = [node for node in self.all_nodes if name == node.name]
        return filtered

    def __get_resources_by_kind(self, kind: str):
        logger.info('Getting all %s', kind)
        filtered = [
            node for node in self.all_nodes if kind.capitalize() == node.kind
        ]
        return filtered

    def __get_resource(self, name: str, kind: str):
        logger.info('Getting resource %s of kind %s', name, kind)
        for node in self.all_nodes:
            if node.kind == kind.capitalize() and node.name == name:
                return node

    def __get_application(self, app_name: str) -> Application:
        endpoint = self.server + f'/api/v1/applications/{app_name}'
        logger.info('Getting application %s', app_name)
        response = self.__get(
            endpoint,
            self.header,
        )
        if response.status_code == 200:
            application = Application(**response.json())
            self.__application = application
        else:
            error = DefaultError(**response.json())
            logger.error(
                "Status code: %i, Message: %s",
                error.code,
                error.message,
            )
            raise ArgoCDAPIError(error.error)

    def get_degraded_resources(self, app_name: str):
        logger.info('Getting degraded resources for %s', app_name)
        resources = self.__get_application(app_name).status.resources
        degraded = list[DegradedResource]
        for resource in resources:
            if resource.status and (
                resource.status != 'Synced' or resource.health != 'Healthy'
            ):
                degraded.append(
                    DegradedResource(
                        name=resource.name,
                        sync=resource.status,
                        health=resource.health.status,
                        message=resource.health.message,
                    )
                )
