import os
import boto3
import uuid

from bat.core.logger import logger
from bat.core.errors import (
    AwsConfigurationError,
    BucketNotFound,
    BucketConfigurationError,
    NoAvailableEvents,
)


class S3:
    def __init__(self, bucket: str, profile: str = "default") -> None:
        logger.info("Initializing s3 client")
        session = boto3.Session(profile_name=profile)
        self.s3 = session.client('s3')
        self.bucket = bucket

        self.__validate(session)
        logger.info("AWS Credentials Validation Complete")

    def __validate(self, session: boto3.Session) -> None:
        sts = session.client('sts')
        try:
            sts.get_caller_identity()
        except Exception as e:
            raise AwsConfigurationError from e
        try:
            self.s3.list_objects_v2(Bucket=self.bucket)
        except Exception as e:
            raise BucketNotFound(self.bucket) from e

    def get_all_objects(self) -> list:
        try:
            resp = self.s3.list_objects_v2(Bucket=self.bucket)
        except Exception as e:
            raise e

        objects = []
        try:
            for obj in resp['Contents']:
                objects.append(obj)
            logger.info(f"Retrieved all objects in s3 bucket: {self.bucket}")
        except Exception:
            logger.error("Bucket is empty", exc_info=True)
            raise BucketConfigurationError(self.bucket)

        return objects

    def get_root_folder_structure(self) -> list:
        objects = self.get_all_objects()
        root = set()
        for obj in objects:
            root.add(obj['Key'].split('/')[0])
        root = list(root)
        logger.info(f"Retrieved root folder structure in {self.bucket}")
        root.sort()
        return root

    def get_all_notifications(self):
        try:
            events = self.s3.get_bucket_notification_configuration(
                Bucket=self.bucket
            )
        except Exception:
            logger.critical(
                "No bucket found by the name: {}".format(self.bucket),
                exc_info=True,
            )
        return events

    def get_topic_events(self):
        events = self.get_all_notifications()
        try:
            events = events.get('TopicConfigurations')
        except Exception:
            logger.error(
                "No Topic event notifications available", exc_info=True
            )
            raise NoAvailableEvents("TopicConfigurations")

        return events

    def get_queue_events(self):
        events = self.get_all_notifications()
        try:
            events = events.get('QueueConfigurations')
        except Exception:
            logger.error(
                "No Queue event notifications available", exc_info=True
            )
            raise NoAvailableEvents("QueueConfigurations")
        return events

    def get_lambda_events(self):
        events = self.get_all_notifications()
        try:
            events = events.get('LambdaFunctionConfigurations')
        except Exception:
            logger.error(
                "No Lambda Function event notifications available",
                exc_info=True,
            )
            raise NoAvailableEvents("LambdaFunctionConfigurations")
        return events

    def upload_file(self, local_path: str, object_path: str):
        try:
            logger.info(
                'Uploading %s to %s:%s', local_path, self.bucket, object_path
            )
            self.s3.upload_file(
                Filename=local_path, Bucket=self.bucket, Key=object_path
            )
            logger.info(
                "Finished uploading %s to %s:%s",
                local_path,
                self.bucket,
                object_path,
            )
        except Exception as e:
            raise e

    def rename_and_upload(
        self, category: str, local_path: str, object_path: str
    ):
        logger.info('Renaming asset...')
        name = uuid.uuid4()
        oldfile = os.path.basename(local_path)
        ext = oldfile.split('.')[-1]
        newfile = f"{name}.{ext}"
        object_path = os.path.join(os.path.dirname(object_path), newfile)
        self.upload_file(local_path, object_path)

        return newfile

    def download_file(self, local_path: str, object_path: str):
        try:
            logger.info('Downloading object from s3: %s', object_path)
            self.s3.download_file(
                Bucket=self.bucket, Key=object_path, Filename=local_path
            )
        except Exception as e:
            raise e
