import os
import time
import uuid
import json
import requests

from typing import Optional
from datetime import datetime

from bat.core.definitions import CONFIGS_DIR, ASSETS_DIR
from bat.libs.kubernetes import K8sClientConn
from bat.libs.pubnub import Pubnub
from bat.core.logger import logger
from bat.libs.playlist import Scheduler
from bat.core.errors import ServerError, MapsorError, PollingTimeout


class Cloudport:
    def read_config(self) -> dict:
        path = os.path.join(CONFIGS_DIR, "test-lock.json")
        with open(path, 'r') as file:
            config = json.load(file)
        self.kubectl = K8sClientConn(config['platform_config']['cluster'])
        secret_ns = f"{config['blip']['customer']}-automation"
        admin = self.kubectl.extract_secret_data(
            "automation-admin-secret", secret_ns
        )
        admin = json.loads(admin['admin'])
        config = {
            'customer': config['blip']['customer'],
            'token': admin['token'],
            'blip-url': f"https://{config['blip']['customer']}.{config['blip']['hosted_zone']}",
            'media': config['scenario']['media_asset'],
            'graphics': config['scenario']['graphics_asset'],
            'timezone': config['scenario']['timezone'],
            'channel': config['channel']['code'],
            'channel_name': config['channel']['name'],
            'live-url': f"https://{config['blip']['customer']}-vision.{config['blip']['hosted_zone']}",
        }

        return config


class Blip(Cloudport):
    def __init__(self, config: Optional[dict] = None) -> None:
        self.config = super().read_config() if not config else config
        self.feed_id = self.__get_feed_id(self.config['channel'])

    def __get_feed_id(self, channel):
        endpoint = f"{self.config['blip-url']}/v1/api/feeds"
        params = {"token": self.config['token'], "channel_code": channel}
        try:
            resp = requests.get(url=endpoint, params=params)
            if resp.status_code != 200:
                logger.error("Received a %i status code", resp.status_code)
                logger.error("Response: %s", resp.json())
                raise ServerError(endpoint, resp.status_code)
            resp = resp.json()
            if len(resp['feeds']) == 0:
                logger.error(
                    "No feed found for channel code %s", self.config['channel']
                )
                raise Exception(
                    f"No feed found for channel code: {self.config['channel']}"
                )

            return resp['feeds'][0]['feed_id']
        except Exception as e:
            logger.error("Cannot reach blip", exc_info=True)
            raise ServerError(endpoint) from e

    def poll_asset_state(
        self, category: str, timeout_min: int = 30, poll_exp: float = 2.0
    ) -> Optional[bool]:
        endpoint = f"{self.config['blip-url']}/v1/api/media"
        params = {
            "token": self.config['token'],
            "feed_id": self.feed_id,
            "category": category,
        }
        poll_num = 1
        total_sleep = 1
        state = ""
        asset = self.config[category].split('.')[0]
        while total_sleep < timeout_min * 60:
            logger.debug("Polling %s asset state \t%ss", asset, total_sleep)
            time.sleep(pow(poll_num, poll_exp))
            total_sleep += pow(poll_num, poll_exp)
            poll_num += 1
            try:
                resp = requests.get(url=endpoint, params=params)
                if resp.status_code != 200:
                    logger.error("Received a %i status code", resp.status_code)
                    raise ServerError(endpoint, resp.status_code)
                resp = resp.json()
                for media in resp['media']:
                    if media['asset_id'] == asset:
                        state = media['state']
                        if state == 'queued' or state == 'new':
                            logger.info('Asset Status: %s', state)
                        if state == 'submitted':
                            logger.info('Asset Status: Submitted to Mapsor')
                        if state == 'uploaded':
                            logger.info("Mapsor finished processing")
                            return True
                        if (
                            state == 'failed'
                            or state == 'qc_failed'
                            or state == "invalid"
                            or state == 'meta_absent'
                        ):
                            logger.error(
                                "Mapsor asset processing failed with info: %s",
                                state,
                            )
                            return False
                        else:
                            logger.debug("Asset state: %s", state)
            except Exception as e:
                logger.error("Cannot reach blip", exc_info=True)
                raise ServerError(endpoint) from e

        if state == "queued":
            logger.error("Could not connect to Mapsor")
            raise MapsorError(category)
        if state == "submitted":
            logger.error(
                "Mapsor processing for %s not completed after %i minutes",
                category,
                timeout_min,
            )
            raise PollingTimeout(timeout_min)

    def create_playlist(
        self, type: str, playout_time: datetime, live_name: Optional[str] = None
    ) -> str:
        logger.info("Creating %s playlist from template", type)
        template_path = ASSETS_DIR

        if type != "media" and type != "live":
            raise Exception("type should be one of media or live")

        if type == 'media':
            dist_path = os.path.join(template_path, "dist")
            lockfile = os.path.join(CONFIGS_DIR, 'test-lock.json')

            asset_name = self.config['media'].split(".")[0]
            asset_duration = 10.010
            values = {
                "asset-name": asset_name,
                "asset-time": playout_time,
                "asset-duration": asset_duration,
            }
            template = os.path.join(template_path, "file-playout-template.json")
            show_id = Scheduler().media_writer(template, values)

            # Updating lockfile
            logger.info("Updating test lockfile with event-ids")
            with open(lockfile, 'r') as f:
                loaded = json.load(f)
            loaded['scenario']['media-show-id'] = show_id
            with open(lockfile, 'w') as f:
                json.dump(loaded, f)

            playlist_path = os.path.join(dist_path, "file-playout.json")

        if type == 'live':
            dist_path = os.path.join(template_path, "dist")
            lockfile = os.path.join(CONFIGS_DIR, 'test-lock.json')
            with open(lockfile, 'r') as f:
                loaded = json.load(f)
            live_duration = 30
            if live_name is None:
                live_name = Live(config=self.config).create_live()

            values = {
                "live-name": live_name,
                "live-time": playout_time,
                "live-duration": live_duration,
                # TODO: This will change once ingest setup is implemented
                "ingest-label": f"{loaded['blip']['customer']}-ingest",
                "channel_code": loaded['channel']['code'],
            }
            template = os.path.join(template_path, "live-playout-template.json")
            show_id = Scheduler().live_writer(template, values)

            # Updating lockfile
            logger.info("Updating test lockfile with event-ids")

            loaded['scenario']['live-show-id'] = show_id
            with open(lockfile, 'w') as f:
                json.dump(loaded, f)

            playlist_path = os.path.join(dist_path, "live-playout.json")

        # Upload playlist
        token = self.config['token']
        blip = self.config['blip-url']
        feed_id = self.feed_id
        upload = f"{blip}/v1/api/playout/playlists.json?token={token}&feed_id={feed_id}"
        try:
            with open(playlist_path, 'r') as f:
                data = json.load(f)
            resp = requests.put(upload, json=data)
            logger.debug(resp.json())
            if resp.json().get('error'):
                raise Exception(
                    f"call: {upload}, error: {resp.json()['error']}"
                )
            playlist_id = resp.json()['request_id_to_playlist_id_mappings'][0][
                'id'
            ]

            # ! Poor way of handling this
            if type == 'live':
                playlist_id += 1

            logger.debug("Playlist ID for %s: %i", type, playlist_id)
            if resp.json().get('error'):
                raise Exception(
                    f"call: {upload}, error: {resp.json()['error']}"
                )
            logger.info("Response: %s", resp.json()['message'])
        except Exception as e:
            logger.error("Failed creating playlist")
            raise e

        # Get playlist status
        status = f"{blip}/v1/api/playlist/{playlist_id}?token={token}"
        for i in range(5):
            try:
                data = {"feed_id": self.feed_id}
                resp = requests.get(status, data=data)
                if resp.json()['status'] == 'new':
                    logger.info("Playlist uploaded successfully")
                    break
                elif resp.json()['status'] == 'invalid':
                    logger.error("Uploaded playlist is invalid")
                    raise Exception("Invalid playlist upload")
                else:
                    if i == 4:
                        logger.error("Unable to upload playlist")
                        raise ServerError(status)
            except Exception as e:
                raise e
            time.sleep(10)

        publish = f"{blip}/v1/api/playlist_edit/publish?token={token}&feed_id={feed_id}&playlist_ids[]={playlist_id}&sync=true"

        resp = requests.post(publish)
        logger.info("Response: %s", resp.json()['message'])

        return show_id


class Live(Cloudport):
    def __init__(self, config: Optional[dict] = None) -> None:
        self.config = super().read_config() if not config else config

    def create_live(self) -> str:
        name = str(uuid.uuid4())
        body = {
            "name": name,
            "mode": "Managed",
            "tz": self.config['timezone'],
            "duration": 30000,
            "event_id_format": "$name-$time",
            "category": "Others",
            "sub_category": "Others",
            "ingest_labels": [f'{self.config["customer"]}-ingest'],
            "intents": [
                {
                    "name": "Event End Break",
                    "duration": 10000,
                    "schedule_type": "unscheduled",
                    "intent_type": "break",
                },
                {
                    "name": "Event Start Break",
                    "duration": 10000,
                    "schedule_type": "unscheduled",
                    "intent_type": "break",
                },
            ],
            "operator": "Live Operator",
            "duration_mode": "fixed",
            "auto_live": True,
            "record": False,
            "immediate_live": True,
        }

        endpoint = f"{self.config['live-url']}/v1/api/templates"
        resp = requests.post(url=endpoint, json=body)
        if resp.status_code != 200:
            logger.error("Received a %i status code", resp.status_code)
            raise ServerError(endpoint, resp.status_code)
        logger.debug("Reponse: %s", resp.json())
        logger.info("Created live template: %s", name)

        return name

    def get_latest_event_id(self):
        endpoint = f"{self.config['live-url']}/v1/api/events"
        try:
            resp = requests.get(endpoint)
            resp = resp.json()
            logger.info(
                "Retrieved latest event: %s", resp['events'][0]['event_id']
            )
            return resp['events'][0]['event_id']
        except Exception as e:
            logger.error("Error in getting latest event id", exc_info=True)
            raise e


class Player(Cloudport):
    def __init__(self, config: Optional[dict] = None) -> None:
        self.config = super().read_config() if not config else config
        self.feed_id = Blip().feed_id

    def get_logs(
        self,
        msg_type: str,
        pubnub_keys: dict,
        from_time: datetime,
        domain: str,
        hosted_zone: str,
        channel: int,
    ):
        blip_url = f"{domain}.{hosted_zone}"
        blip_topic = blip_url.replace('.', '_')

        # TODO: Remove "001" hard coded
        topic = f"player|{blip_topic}|{domain}|{channel}|{channel}_001"
        logger.info("Fetching asset logs for topic: %s", topic)

        pn = Pubnub(
            pubnub_keys['publish'],
            pubnub_keys['subscriber'],
            pubnub_keys['secret'],
        )
        from_time_int = int(datetime.timestamp(from_time) * 10000000)
        logger.debug("Getting pubnub logs from timestamp: %i", from_time_int)
        logs = pn.fetch_messages(topic, from_time_int)
        res = []
        for log in logs:
            if log['message']['msg_type'] == msg_type:
                res.append(log)
        if len(res) == 0:
            logger.warning("Type %s not found in logs")

        return res

    def get_downloaded_assets(self, category: str):
        logger.info("Fetching downloaded assets")
        endpoint = f"{self.config['blip-url']}/v1/api/media"
        params = {
            "token": self.config['token'],
            "feed_id": self.feed_id,
            "category": category,
        }
        for _ in range(20):
            try:
                resp = requests.get(endpoint, params=params)
                if resp.status_code != 200:
                    raise ServerError(endpoint, resp.status_code)
                resp = resp.json()
                asset_path = (
                    f"Media/S3/{self.config['channel']}/{self.config[category]}"
                )
                for media in resp['media']:
                    if asset_path == media['filename']:
                        for channel in media['headends']:
                            # ! Don't hardcode _001
                            if (
                                f"{self.config['channel']}_001"
                                == channel['hcode']
                            ):
                                logger.info(
                                    "Media asset status on Player: %s...",
                                    channel['status'],
                                )
                                if channel['status'] == 'downloaded':
                                    return True
                time.sleep(10)
            except Exception as e:
                raise e

        raise Exception(
            "Asset %s not downloaded on player", self.config[category]
        )
