import json
import base64

from kubernetes import config, client

from bat.core.logger import logger
from bat.core.errors import JobRunning
from bat.models.data import KubernetesSecret
from bat.models.kubernetes import JobConfig


class KubernetesClient:
    def __init__(self, cluster_name: str | None = None):
        self.__validate(cluster_name)
        logger.info('Kubernetes client validated')
        self.core = client.CoreV1Api()
        self.custom = client.CustomObjectsApi()
        self.batch = client.BatchV1Api()

    def __validate(self, name: str | None) -> None:
        try:
            if name:
                contexts, _ = config.list_kube_config_contexts()
                for context in contexts:
                    if name in context['name']:
                        logger.info(
                            'Loading kubeconfig for %s', context['name']
                        )
                        config.load_kube_config(context=context['name'])
                        return
            else:
                logger.info('Loading in-cluster config')
                config.load_incluster_config()
        except Exception as e:
            raise e

    def get_custom_resource(
        self,
        resource: str,
        name: str,
        namespace: str,
        version: str,
        group: str,
    ):
        logger.info('Getting custom resource: %s:%s', resource, name)
        resource = self.custom.get_namespaced_custom_object(
            group=group,
            version=version,
            plural=resource,
            name=name,
            namespace=namespace,
        )

        return json.dumps(resource)

    def get_secret_data(self, secret: str, namespace: str):
        logger.info('Reading secret: %s:%s', namespace, secret)
        data = self.core.read_namespaced_secret(
            secret,
            namespace,
        ).data
        secrets = list[KubernetesSecret]

        for secret, value in data.items():
            decoded = base64.b64decode(value)
            secrets.append(
                KubernetesSecret(
                    name=secrets,
                    value=decoded.decode('utf-8'),
                )
            )
            return secrets

    def create_job(self, config: JobConfig, namespace: str):
        logger.info('Creating new job: %s', config.name)
        containers = []
        for container in config.containers:
            containers.append(
                client.V1Container(
                    name=container.name,
                    image=container.image,
                    image_pull_policy=container.pull_policy,
                    command=container.command,
                    env_from=[
                        client.V1EnvFromSource(
                            config_map_ref=client.V1ConfigMapEnvSource(
                                name=container.config_map,
                            )
                        )
                    ],
                )
            )
        template = client.V1PodTemplateSpec(
            spec=client.V1PodSpec(
                restart_policy=config.restart,
                containers=containers,
                service_account_name=config.service_account,
            )
        )
        spec = client.V1JobSpec(template=template)
        job = client.V1Job(
            api_version=config.version,
            kind="Job",
            metadata=client.V1ObjectMeta(name=config.name),
            spec=spec,
        )

        existing = None
        try:
            existing = self.get_job_status(config.name, namespace)
            if existing.active:
                logger.warning(
                    'Job %s:%s still running', namespace, config.name
                )
                raise JobRunning(name=f'{namespace}:{config.name}')
            _ = self.delete_job(config.name, namespace)
        except JobRunning as e:
            raise e
        except Exception as e:
            logger.warning('Failed getting job status: %s', str(e))
            pass

        retries = 0
        max_retries = 10
        while retries <= max_retries:
            try:
                response = self.batch.create_namespaced_job(
                    body=job, namespace=namespace
                )
                return response
            except Exception as e:
                logger.warning(f"Request failed: {str(e)}")
                retries += 1
                if retries == max_retries:
                    raise e

    def delete_job(self, name: str, namespace: str) -> client.V1Status:
        logger.info('Deleting job: %s:%s', namespace, name)
        status: client.V1Status = self.batch.delete_namespaced_job(
            name=name, namespace=namespace
        )
        return status

    def get_job_status(self, name: str, namespace: str) -> client.V1JobStatus:
        logger.info('Getting job status for - %s:%s', namespace, name)
        status: client.V1JobStatus = self.batch.read_namespaced_job_status(
            name=name, namespace=namespace
        )
        logger.info(status.status)
        return status.status
