from pydantic import BaseModel
from typing import Any, List, Mapping


class JWTRequest(BaseModel):
    password: str
    username: str


class JWTResponse(BaseModel):
    token: str


class DefaultError(BaseModel):
    code: int
    details: list[Mapping[str, str]] | None
    error: str
    message: str


class _ResourcesInfoItem(BaseModel):
    capacity: str
    requestedByApp: str
    requestedByNeighbors: str
    resourceName: str


class _SystemInfo(BaseModel):
    architecture: str
    bootID: str
    containerRuntimeVersion: str
    kernelVersion: str
    kubeProxyVersion: str
    kubeletVersion: str
    machineID: str
    operatingSystem: str
    osImage: str
    systemUUID: str


class _Host(BaseModel):
    name: str
    resourcesInfo: List[_ResourcesInfoItem]
    systemInfo: _SystemInfo


class _Health(BaseModel):
    message: str | None
    status: str


class _InfoItem(BaseModel):
    name: str
    value: str | None


class _Port(BaseModel):
    error: str
    port: int
    protocol: str


class _Ingress(BaseModel):
    hostname: str | None
    ip: str | None
    ports: List[_Port] | None


class _TargetRef(BaseModel):
    group: str | None
    kind: str
    name: str
    namespace: str
    uid: str | None
    version: str | None


class _NetworkingInfo(BaseModel):
    externalURLs: List[str] | None
    ingress: List[_Ingress] | None
    labels: Mapping[str, str] | None
    targetLabels: Mapping[str, str] | None
    targetRefs: List[_TargetRef] | None


class _ParentRef(BaseModel):
    group: str | None
    kind: str
    name: str
    namespace: str
    uid: str
    version: str | None


class _Node(BaseModel):
    createdAt: str
    health: _Health | None
    images: List[str] | None
    info: List[_InfoItem] | None
    networkingInfo: _NetworkingInfo | None
    parentRefs: List[_ParentRef] | None
    group: str | None
    uid: str
    name: str
    kind: str
    version: str | None
    namespace: str
    resourceVersion: str


class ResourceTree(BaseModel):
    hosts: List[_Host] | None
    nodes: List[_Node]
    orphanedNodes: List[_Node]


class _Metadata(BaseModel):
    name: str
    namespace: str
    uid: str
    resourceVersion: str
    generation: int
    creationTimestamp: str
    ownerReferences: Any
    finalizers: List[str]
    managedFields: Any


class _FileParameter(BaseModel):
    name: str
    path: str


class _Helm(BaseModel):
    valueFiles: List[str] | None
    fileParameters: List[_FileParameter] | None


class _Source(BaseModel):
    repoURL: str | None
    path: str | None
    targetRevision: str | None
    helm: _Helm | None


class _Destination(BaseModel):
    server: str
    namespace: str


class _Automated(BaseModel):
    prune: bool | None
    selfHeal: bool | None


class _SyncPolicy(BaseModel):
    automated: _Automated | None
    syncOptions: List[str] | None


class _Spec(BaseModel):
    source: _Source
    destination: _Destination
    project: str
    syncPolicy: _SyncPolicy | None


class _Resource(BaseModel):
    version: str
    kind: str
    namespace: str | None
    name: str
    hook: bool | None
    requiresPruning: bool | None
    status: str | None
    health: _Health | None
    group: str | None


class _ComparedTo(BaseModel):
    source: _Source
    destination: _Destination


class _Sync(BaseModel):
    status: str | None
    comparedTo: _ComparedTo | None
    revision: str


class _HistoryItem(BaseModel):
    revision: str
    deployedAt: str
    id: int
    source: _Source
    deployStartedAt: str


class _Condition(BaseModel):
    type: str
    message: str
    lastTransitionTime: str


class _InitiatedBy(BaseModel):
    automated: bool | None


class _Retry(BaseModel):
    limit: int | None


class _Operation(BaseModel):
    sync: _Sync | None
    initiatedBy: _InitiatedBy | None
    retry: _Retry | None


class _SyncResult(BaseModel):
    resources: List[_Resource]
    revision: str
    source: _Source


class _OperationState(BaseModel):
    operation: _Operation
    phase: str
    message: str
    syncResult: _SyncResult
    startedAt: str
    finishedAt: str


class _Summary(BaseModel):
    externalURLs: List[str] | None
    images: List[str] | None


class _Status(BaseModel):
    resources: List[_Resource]
    sync: _Sync
    health: _Health
    history: List[_HistoryItem]
    conditions: List[_Condition] | None
    reconciledAt: str
    operationState: _OperationState
    sourceType: str
    summary: _Summary | None


class Application(BaseModel):
    metadata: _Metadata
    spec: _Spec
    status: _Status


class DegradedResource(BaseModel):
    name: str
    sync: str
    health: str
    message: str | None
