import os
import atexit
import shutil
import logging
import tempfile

from pathlib import Path
from typing import Literal
from functools import lru_cache
from abc import ABC, abstractmethod
from bat.core.errors import ImproperTestConfiguration

from bat.libs.github import Github
from bat.models.config import Config, ConfigMap
from bat.core.definitions import CONFIGS_DIR, TEST_CONFIG
from bat.models.cp_release import Release as CpRelease
from bat.validators.base_config import (
    ArgoCDConfig,
    PlatformConfig,
    PlayoutConfig,
    Release,
    TestConfig,
)
from bat.libs.utils import read_yaml, write_json, cached_read_json
from bat.validators.deploy_config import AccountConfig, BlipConfig

logger = logging.getLogger()


@lru_cache
def get_working_dir(repo: str, user: str, password: str, org: str, branch: str):
    logger.info('Creating temporary working directory')
    tempdir = tempfile.mkdtemp()
    atexit.register(shutil.rmtree, tempdir)
    logger.info('Cloning deploy repo %s', repo)
    git_handle = Github(
        repo_name=repo,
        username=user,
        password=password,
        working_dir=tempdir,
        org=org,
    )
    git_handle.clone(branch=branch)
    return tempdir


class Setup(ABC):
    def __init__(self, config_path: Path = TEST_CONFIG) -> None:
        self.__config = Config(**cached_read_json(config_path))

    @abstractmethod
    def create_template(self, platform_release, cp_release: CpRelease):
        raise NotImplementedError

    @abstractmethod
    def git_deploy(self, template):
        raise NotImplementedError

    @property
    def working_dir(self):
        git = self.test_config.git_config
        return get_working_dir(
            git.repo,
            git.user,
            git.password,
            git.org,
            git.branch,
        )

    @property
    def test_config(self):
        return self.__config.base_config

    @property
    def git_config(self):
        return self.test_config.git_config

    @property
    def blip_config(self):
        return self.__config.deploy_blip

    @property
    def account_config(self):
        return self.__config.deploy_account

    def read_template(self, release: Release, model: object = None):
        git_handle = Github(
            repo_name=release.repo,
            username=self.git_config.user,
            password=self.git_config.password,
            working_dir=self.working_dir,
            org=self.git_config.org,
        )
        git_handle.clone(branch=release.version)
        crd_path = Path(
            os.path.join(self.working_dir, release.repo, release.template_path)
        )
        logger.info("Reading CRD for Release: %s", release.version)
        parser = 'pyyaml' if model else 'ruamel'
        loaded = read_yaml(crd_path, parser)
        loaded = model(**loaded) if model else loaded
        return loaded

    def update_docker_image(self, old: str, new: str, replace_in: str):
        old = old.split(':')[-1]
        new = new.split(':')[-1]
        replace_in.replace(old, new)
        return replace_in

    @staticmethod
    def get_region_short(region: str, cloud: Literal['aws', 'gcp']) -> str:
        region_short = {
            "us-east-1": "use1",
            "us-east-2": "use2",
            "us-west-1": "usw1",
            "us-west-2": "usw2",
            "eu-north-1": "eun1",
            "eu-west-1": "euw1",
            "eu-west-3": "euw3",
            "eu-west-2": "euw2",
            "eu-south-1": "eus1",
            "eu-central-1": "euc1",
            "ap-south-1": "as1",
            "ap-northeast-1": "ane1",
            "ap-northeast-2": "ane2",
            "ap-northeast-3": "ane3",
            "ap-east-1": "ae1",
            "ap-southeast-1": "ase1",
            "ap-southeast-2": "ase2",
            "ap-southeast-3": "ase3",
            "af-south-1": "afs1",
            "me-south-1": "mes1",
            "sa-east-1": "sae1",
            "ca-central-1": "cac1",
        }
        if cloud == 'aws':
            return region_short[region]
        elif cloud == 'gcp':
            ...
        else:
            raise NotImplementedError


class SetupInit:
    def __init__(self) -> None:
        logger.info('Starting test setup')
        self.test_config = Path(os.path.join(CONFIGS_DIR, 'test.yaml'))
        self.blip_config = Path(os.path.join(CONFIGS_DIR, 'blip.yaml'))
        self.account_config = Path(os.path.join(CONFIGS_DIR, 'account.yaml'))
        self.adh_config = Path(os.path.join(CONFIGS_DIR, 'adh.yaml'))

        base_config = self.__validate_base_config()
        blip = None
        account = None
        if base_config.deploy_blip:
            blip = self.__validate_deploy_config(BlipConfig, self.blip_config)
        if base_config.deploy_account:
            account = self.__validate_deploy_config(
                AccountConfig, self.account_config
            )

        self.generate_lockfile(base_config, blip, account)

    def __validate_base_config(self) -> TestConfig:
        try:
            config = read_yaml(self.test_config)
            test_config = TestConfig(**config)
        except Exception as e:
            raise ImproperTestConfiguration('Invalid test configuration') from e
        logger.info('Validated base config')
        return test_config

    def __validate_deploy_config(
        self, validator: BlipConfig | AccountConfig, path: Path
    ):
        try:
            config = read_yaml(path)
            deploy_config = validator(**config)
        except FileNotFoundError as e:
            raise ImproperTestConfiguration(
                f'{validator.__name__} not found'
            ) from e
        except Exception as e:
            raise ImproperTestConfiguration('Invalid test configuration') from e
        logger.info('Validated %s deploy config', validator.__name__)
        return deploy_config

    def generate_lockfile(
        self,
        base_config: TestConfig,
        deploy_blip: BlipConfig | None,
        deploy_account: AccountConfig | None,
    ):
        logger.info('Generating lockfile...')
        config = Config(
            base_config=base_config,
            deploy_account=deploy_account,
            deploy_blip=deploy_blip,
        )
        if config.base_config.exec_mode == 'api':
            try:
                env_data = ConfigMap()
                config.base_config.allure_server = env_data.allure_server
                config.base_config.platform_config = PlatformConfig(
                    enterprise=env_data.enterprise,
                    environment=env_data.environment,
                    cp_integration=env_data.cp_integration,
                    cp_orchestration=env_data.cp_orchestration,
                    cluster=env_data.cluster,
                    cloud=env_data.cloud,
                )
                config.base_config.argocd_config = ArgoCDConfig(
                    server=env_data.argocd_server,
                    username=env_data.argocd_username,
                    password=env_data.argocd_password,
                )
                config.base_config.playout_config = PlayoutConfig(
                    media_asset=env_data.media_asset,
                    graphics_asset=env_data.graphics_asset,
                    timezone=env_data.playout_timezone,
                )
                logger.info(
                    'Setting all deploy to false since this is running in API mode'
                )
                config.base_config.deploy_account = False
                config.base_config.deploy_blip = False
                config.base_config.deploy_player = False
            except Exception as e:
                raise ImproperTestConfiguration(
                    'ConfigMap environment variables not set correctly'
                ) from e

        write_json(TEST_CONFIG, config.dict())
