from pathlib import Path
from typing import Literal

from bat.libs.bucket import S3
from bat.models import TestData
from bat.core.logger import logger
from bat.setups._blip import BlipSetup
from bat.libs.argocd import ArgoCDClient
from bat.setups._config import SetupConfig
from bat.setups._player import PlayerSetup
from bat.setups._account import AccountSetup
from bat.core.definitions import TEST_CONFIG
from bat.libs.kubernetes import KubernetesClient


class SetupFactory:
    def __init__(
        self,
        upto: Literal['infra', 'blip', 'account', 'channel', 'live', 'playout']
        | None = None,
        config_path: Path = TEST_CONFIG,
    ) -> None:
        self.__setup_config = SetupConfig(config_path).test_config
        self.config_path = config_path

        logger.info('Running setup upto: %s', upto)
        if upto:
            call_upto: callable = f"_SetupFactory__setup_{upto}"
            getattr(self, call_upto)()

        logger.info('Setting up kubernetes handler')
        self.__kubernetes_handler = KubernetesClient(
            self.__setup_config.platform_config.cluster
            if self.__setup_config.exec_mode == 'cli'
            else None
        )
        logger.info('Setting up argocd handler')
        argocd_config = self.__setup_config.argocd_config
        self.__argocd_handler = ArgoCDClient(
            argocd_config.server,
            argocd_config.username,
            argocd_config.password,
        )

    def __setup_infra(self):
        self.__test_data = TestData(
            platform_config=self.__setup_config.platform_config
        )

    def __setup_blip(self):
        self.__setup_infra()
        self.__test_data.blip_data = BlipSetup(self.config_path).data
        self.__test_data.blip_data.existing = (
            not self.__setup_config.deploy_blip
        )

    def __setup_account(self):
        self.__setup_blip()
        self.__test_data.account_data = AccountSetup(
            self.test_data.blip_data,
            self.config_path,
        ).data
        self.test_data.account_data.existing = (
            not self.__setup_config.deploy_account
        )
        if self.__setup_config.platform_config.cloud == 'aws':
            self.__bucket_handler = S3(
                self.test_data.account_data.bucket,
                self.__setup_config.platform_config.profile,
            )
        else:
            ...

    def __setup_channel(self):
        self.__setup_account()
        self.__test_data.player_data = PlayerSetup(
            self.test_data.blip_data,
            self.test_data.account_data,
            self.config_path,
        )
        self.test_data.player_data.existing = (
            not self.__setup_config.deploy_player
        )

    @property
    def test_data(self):
        return self.__test_data

    @property
    def argocd(self):
        return self.__argocd_handler

    @property
    def kubernetes(self):
        return self.__kubernetes_handler

    @property
    def bucket(self):
        return self.__bucket_handler
