from typing import Literal
from bat.core.errors import ImproperTestConfiguration
from pydantic import BaseModel, Field, root_validator


class GitConfig(BaseModel):
    repo: str
    user: str
    password: str
    org: str
    branch: str | None = "main"


class ArgoCDConfig(BaseModel):
    server: str
    username: str
    password: str


class FileConfig(BaseModel):
    blip_file: str | None = "blip.yaml"
    account_file: str | None = "account.yaml"


class PlatformConfig(BaseModel):
    enterprise: bool
    environment: Literal['prod', 'dev']
    cp_orchestration: bool
    cp_integration: bool
    cluster: str | None
    cloud: Literal['aws', 'gcp']
    stage: str | None
    region: str | None
    profile: str | None


class PlayoutConfig(BaseModel):
    media_asset: str | None = 'e2e-file.mxf'
    graphics_asset: str | None = 'e2e-graphic.zip'
    timezone: str | None = 'US/Eastern'


class Release(BaseModel):
    version: str
    repo: str
    template_path: str


class TestConfig(BaseModel):
    exec_mode: Literal['cli', 'api']
    allure_server: str | None
    git_config: GitConfig | None
    deploy_blip: bool
    deploy_account: bool
    deploy_player: bool
    argocd_config: ArgoCDConfig | None
    existing_blip: str | None
    existing_account: str | None
    existing_channel: str | None
    file_names: FileConfig | None = Field(default_factory=lambda: FileConfig())
    platform_config: PlatformConfig | None
    playout_config: PlayoutConfig | None = Field(
        default_factory=lambda: PlayoutConfig()
    )
    platform_release: Release | None
    cp_release: Release | None

    @root_validator(pre=False)
    def exec_validation(cls, values):
        deploy_one: bool = (
            values.get('deploy_blip')
            or values.get('deploy_account')
            or values.get('deploy_player')
        )

        if deploy_one and (
            not values.get('platform_release') or not values.get('cp_release')
        ):
            raise ImproperTestConfiguration(
                'deploy enabled but releases not given'
            )

        if values.get('exec_mode') == 'cli' and (
            not values.get('git_config')
            or not values.get('platform_config')
            or not values.get('argocd_config')
        ):
            raise ImproperTestConfiguration(
                'git config, platform config and argocd config need to be provided for cli execution mode'
            )

        return values
